function [t_s_u,t_s_r] = SpikegenerationLIF(Ntr,steps, dt)

% This Matlab code performs the following

% 1. Setting parameter
% 2. Generation of the bimodal input distribution per time step
% 3. Initialization values
% 4. Calculating random numbers in general/ setting seeds
     % Note, for given spike train (Spike.mat) we use 'stream_uncommon_seed=1' and
     % 'stream_common_seed = 1'
% 5. Spike generation with LIF model with reset
     % t_s_u are spike trains with varying stimuli generated
     % t_s_r are spike trains with fix stimulus generated
     

%% 1. Parameter
    

% Time-vector:
T_start = 0; % (ms)


T_SteadyState =  0;%100; %(ms) %1000; %(ms)


snr =     0.5; % Strength of repeated input. No rep = 0, All rep = 1;

% Leaky Integragtor:
LI_Tau_mem = 10;  % Membrane time constant (ms)
LI_equi    = 0;   % Equilibrium potential (ms)
LI_R       = 1;   % Resistance for Input stimulus 
                  

% Spike Generation:
ST_threshold = LI_equi+3;
LI_V_reset   = LI_equi-3;



%% 2. Generation of the bimodal input distribution per time step:

ng_a1 = -15;
ng_a2 =  -5;

ng_b1 =   5;
ng_b2 =   7;


ng_xmin  = - 100;
ng_xmax  =   100;
ng_steps = 1000;

[ng_probdis,ng_x] = bimodal_input_distribution(ng_a1,ng_a2,ng_b1,ng_b2,ng_xmin,ng_xmax,ng_steps);


%% 3. Initialization values

%% Starting values;


% Leaky Integrator:
LI_V_start = LI_V_reset; % Setting equilibrium as start



%% Assigning stimestep count and time vector, Help functions, Vectors

%% Assign Help functions

%Decays for OU and LI process
LI_decay1 = exp(-dt/LI_Tau_mem);
LI_decay2 = sqrt(1/LI_Tau_mem/2*(1-LI_decay1^2));




%% Assign Vectors


 t_s_u         = cell(Ntr,1); 
 t_s_r         = cell(Ntr,1);

 


 
 t = zeros(1,steps);

for i = 1:steps
    t(i) = T_start+i*dt;
end



%% 4. Calculating random numbers in general/ setting seeds

% for given spike train we use 'stream_uncommon_seed=1' and
% 'stream_common_seed = 1'

  stream_uncommon_seed = 1;
  stream_uncommon = RandStream('mrg32k3a','Seed',stream_uncommon_seed);
  stream_uncommon_state = stream_uncommon.State;
 
  stream_common_seed = 1;
  stream_common = RandStream('mt19937ar','Seed',1); 
  stream_common_state = stream_common.State;


%% 5. Spike generation with LIF model with reset


    
 % Assign Starting values
    LI_V_u  = ones(1,Ntr)*LI_V_start;
    LI_V_r  = ones(1,Ntr)*LI_V_start;
 
 % Assign RandomStreams
    %Uniques
    set(stream_uncommon,'Substream',1);
    %Repeats
    stream_common = RandStream('mt19937ar','Seed',0);
    stream_common.State = stream_common_state;
    
    t_time = T_start;
 for k = 1:steps;

    t_time = t_time+dt;

    % Input-Processes
    ng_tn = ng_x(randp_randstream(ng_probdis,rand(stream_uncommon, 1,Ntr)));
    ng_n  = ng_x(randp_randstream(ng_probdis,rand(stream_uncommon, 1,1)));
    ng_stn = ng_x(randp_randstream(ng_probdis,rand(stream_uncommon, 1,Ntr)));


    
% Leaky integrate with reset
                
        LI_V_u = LI_V_u*LI_decay1+LI_decay2*(LI_R*...
                (sqrt(snr)*ng_stn            +sqrt(1-snr)*ng_tn));

        LI_V_r = LI_V_r*LI_decay1+LI_decay2*(LI_R*...
                (sqrt(snr)*repmat(ng_n,1,Ntr)+ sqrt(1-snr)*ng_tn));

% If you prefer, you can use these commented equations that implement the
% more familiar Euler method, they lead to the same results:
%         LI_V_u = LI_V_u + dt/LI_Tau_mem*(-LI_V_u + LI_R/sqrt(dt)*...
%                 (sqrt(snr)*ng_stn            +sqrt(1-snr)*ng_tn));
%         LI_V_r = LI_V_r + dt/LI_Tau_mem*(-LI_V_r + LI_R/sqrt(dt)*...
%                (sqrt(snr)*repmat(ng_n,1,Ntr)+ sqrt(1-snr)*ng_tn));
%     

   % Spikes of spike trains generated with varying stimuli
   spike_yn_u    = (LI_V_u>=ST_threshold);
   index         = find(spike_yn_u);
   
   % Reset
   LI_V_u(index) = LI_V_reset; 
    
 

 if(t_time >= T_SteadyState)
  for i = index
   t_s_u{i} = [t_s_u{i} (t_time-T_SteadyState)]; 
  end
 end
   
   
      % Spikes of spike trains generated with fixed stimulus
   spike_yn_r = (LI_V_r>=ST_threshold);
   index = find(spike_yn_r);
   
   % Reset
   LI_V_r(index) = LI_V_reset;
   
 if(t_time >= T_SteadyState)  
  for i = index
   t_s_r{i} = [t_s_r{i} (t_time-T_SteadyState)]; 
  end
 end 
 
 end
 





end