%% Save directory
dir_name = [dir_return, '/Simulation_Data/ISI/'];


ou_Tau_cor_vec = [0.2 0.5 1 2 10 20 50 100 200];

ST_thresh_vec  = [0.0279 0.063873 0.1163 0.205 0.6538 1.006 1.6925 2.446 3.483];

dt_vec         = [0.01 0.01 0.05 0.05 0.05 0.05 0.05 0.05 0.05];

Dataset_count = length(ou_Tau_cor_vec);

for xx = 1:Dataset_count
clearvars -except xx ou_Tau_cor_vec dt_vec ST_thresh_vec dir_* Dataset_count

%% Parameter

tic_xx = tic;


LI_Tau_mem = 10;

% Number of stimulus repeats
N = 8;

% Trial count
Ntr = 200000;

% Time-vector:
T_start = 0; % (ms)
T_sequence = 10000; % (ms)




% OU-Process:
ou_Tau_cor  = ou_Tau_cor_vec(xx);%0.5;     % (ms) Correlation time of the OU-Process
ou_diffu_const = 1; % Determines how strong the random numbers influence the process
ou_sigma = sqrt(ou_diffu_const); % Sigma here is the standard deviation
ou_mean  = 0;

ou_cor_n =     0; % No noise correlation, noise is always drawn randomly
ou_cor_c =     0; % Independent stimuli for signal entropy trials
ou_cor_r =     0.6; % Cross trial correlation.

% Leaky Integragtor:
LI_equi    = 0;    % Equilibrium potential (ms)
LI_R       = 1;    % Resistance for Input stimulus
                   
if(LI_Tau_mem <= ou_Tau_cor)
   disp('Tau_mem is smaller than Tau_ou!!!!!!')
end


ST_threshold = ST_thresh_vec(xx);
 
LI_V_reset = -ST_threshold;
 


%% Starting values;
%OU-Process:
ou_start = 0;
% Leaky Integrator:
LI_V_start = LI_equi; % Setting equilibrium as start




%% Assigning timestep size, stimestep count and time vector
   dt = dt_vec(xx);% ou_Tau_cor/T_NperTau;
steps = ceil(T_sequence/dt);

t = zeros(1,steps);

for i = 1:steps
    t(i) = T_start+(i-1)*dt;
end

T_seq_korrigiert = t(end)-t(1);




%% Assign Help functions

%Decays for OU and LI process
ou_decay = exp(-dt/ou_Tau_cor);
LI_decay = exp(-dt/LI_Tau_mem);




%% Assign Vectors
% Spike times uniques
 t_s_u       = cell(N,1);
for n = 1:N
  t_s_u{n} = cell(Ntr,1);
end
 
 % Spike times repeats
 t_s_r       = cell(N,1);
for n = 1:N
  t_s_r{n} = cell(Ntr,1);
end

 % Current I at spike (uniques)
 I_s_signal_u       = cell(N,1);
 for n = 1:N
  I_s_signal_u{n} = cell(Ntr,1);
 end
 
 
 % Current I at spike (repeats) 
 I_s_noise_u       = cell(N,1);
 for n = 1:N
  I_s_noise_u{n} = cell(Ntr,1);
 end

%% Calculating processes, setting random seeds
stream_uncommon = RandStream('mrg32k3a','Seed',4);
stream_uncommon_state = stream_uncommon.State;
    
stream_common = RandStream('mt19937ar','Seed',4); 
stream_common_state = stream_common.State;


%% Calculation
tic
parfor n = 1:N
tic_n = tic;
 % Assign Starting values
    signal_u = ones(1,Ntr)*ou_start;
    noise_u  = ones(1,Ntr)*ou_start;  
    signal_r  =            ou_start;
   
    
    LI_V_u  = ones(1,Ntr)*LI_V_start;
    LI_V_r  = ones(1,Ntr)*LI_V_start;
 % Assign RandomStreams
    %Uniques
    set(stream_uncommon,'Substream',n);
    %Repeats
    stream_common = RandStream('mt19937ar','Seed',0);
    stream_common.State = stream_common_state;
  t_time = 0;  
 for k = 1:length(t)
 t_time = t_time + dt;


    % OU-Processes
    gauss_signal_u = randn(stream_uncommon, 1,Ntr);
       signal_u = signal_u*ou_decay+ou_mean*(1-ou_decay)+...
               ou_sigma*sqrt((ou_Tau_cor/2)*(1-ou_decay^2))*gauss_signal_u;
    
    gauss_noise = randn(stream_uncommon,1,Ntr);
       noise_u  = noise_u*ou_decay+ou_mean*(1-ou_decay)+...
               ou_sigma*sqrt((ou_Tau_cor/2)*(1-ou_decay^2))*gauss_noise;
   
           
           
    gauss_signal_r  = randn(stream_uncommon, 1,1);
       signal_r  = signal_r*ou_decay+ou_mean*(1-ou_decay)+...
               ou_sigma*sqrt((ou_Tau_cor/2)*(1-ou_decay^2))*gauss_signal_r;
             
           


   % Leaky integrate
        LI_V_u = LI_V_u*LI_decay+(1-LI_decay)*(LI_equi+LI_R*...
                    (sqrt(ou_cor_r)*signal_u+sqrt(1-ou_cor_r)*noise_u));
                
        LI_V_r = LI_V_r*LI_decay+(1-LI_decay)*(LI_equi+LI_R*...
                    (sqrt(ou_cor_r)*signal_r + sqrt(1-ou_cor_r)*noise_u));


   % Spikes Uniques
   spike_yn_u    = (LI_V_u>=ST_threshold);
   index         = find(spike_yn_u);
   % Reset
   LI_V_u(index) = LI_V_reset;
  



   
  for i = index
   t_s_u{n}{i} = [t_s_u{n}{i} t_time]; 
   
   Current = signal_u(i);
   I_s_signal_u{n}{i} = [I_s_signal_u{n}{i} Current];
   Current = noise_u(i);
   I_s_noise_u{n}{i} = [I_s_noise_u{n}{i} Current];
  end
  
     
   % Spikes Repeats
   spike_yn_r = (LI_V_r>=ST_threshold);
   index = find(spike_yn_r);
   % Reset
   LI_V_r(index) = LI_V_reset;
   
  for i = index
   t_s_r{n}{i} = [t_s_r{n}{i} t_time];
 
   
   
  end
  

 end
 
 disp(strcat(['Stimulus ', num2str(n),' of ', num2str(N),' done. Time was ',num2str(toc(tic_n)), ' seconds.']));       
          

 
end
toc

%% Changing order of trials and neurons in the t_s-cells

% Spike times uniques
 t_s_u_new       = cell(N,Ntr);
% Spike times repeats
 t_s_r_new       = cell(N,Ntr);

for i = 1:N
for j = 1:Ntr
  t_s_u_new{i,j} = t_s_u{i}{j};
  t_s_r_new{i,j} = t_s_r{i}{j};
end
end
  t_s_u = t_s_u_new;
  t_s_r = t_s_r_new;
  
clear t_s_u_new t_s_r_new

%% Changing order of trials and neurons in the t_s-cells

% Spike times uniques
 I_s_u_new       = cell(N,Ntr);
% Spike times repeats
 I_s_r_new       = cell(N,Ntr);

for i = 1:N
for j = 1:Ntr
  I_s_u_new{i,j} = I_s_signal_u{i}{j};
  I_s_r_new{i,j} = I_s_noise_u{i}{j};
end
end
  I_s_signal_u = I_s_u_new;
  I_s_noise_u = I_s_r_new;
  
clear I_s_u_new I_s_r_new

%% Calculating Firing Rates
FiR_u = zeros(N,Ntr);
FiR_r = zeros(N,Ntr);
for i = 1:N
for j = 1:Ntr
   FiR_u(i,j) = length(t_s_u{i,j})/T_seq_korrigiert;
   FiR_r(i,j) = length(t_s_r{i,j})/T_seq_korrigiert;
end
end
FiR_u_mean = mean(mean(FiR_u,2),1);
FiR_r_mean = mean(mean(FiR_r,2),1);




%% Save
R = dir(dir_name);
if (isempty(R))
    mkdir(dir_name);
    R = dir(dir_name);
end

folder_name = [dir_name,'Dataset_',num2str(xx)];

mkdir(folder_name);
cd(folder_name);


simu_time = toc(tic_xx);

save('params.mat','stream_uncommon_state', 'stream_common_state',...
     'Ntr', 'N','t','T_*', 'ou_*', 'LI_*', 'ST*', 'dt');

save('data.mat','t_s_*','I_s_*', 'FiR*','simu_time','-v7.3');

end