

cd(dir_data);


Dataset_folders = dir('Dataset*');
Dataset_count = length(Dataset_folders);


for rr = 1:Dataset_count
clearvars -except rr Dataset_folders dir_* Dataset_count
    
    
cd(dir_data)
cd(Dataset_folders(rr).name)    

% Loading parameter from run:
load('params.mat');
load('data.mat');
load('p_dist.mat');

%% Parameter
% number of stimulus repeats
N = 32*10;

% Trial count
Ntr = 200000;

% Time-vector:
T_start = 0; % (ms)
T_sequence = 500; % (ms)

if(LI_Tau_mem <= ou_Tau_cor)
   disp('Tau_mem is smaller than Tau_ou!!!!!!')
end



%% Starting values;
%OU-Process:
ou_start = 0;
% Leaky Integrator:
LI_V_start = LI_V_reset;




%% Assigning timestep size, stimestep count and time vector
   % dt = 0.05;
steps = ceil(T_sequence/dt);

t = zeros(1,steps);

for i = 1:steps
    t(i) = T_start+(i-1)*dt;
end

T_seq_korrigiert = t(end)-t(1);




%% Assign Help functions

%Decays for OU and LI process
ou_decay = exp(-dt/ou_Tau_cor);
LI_decay = exp(-dt/LI_Tau_mem);


A_signal = sqrt(ou_cor_r);
A_noise  = sqrt(1-ou_cor_r);



%% Assign Vectors
% Spike times uniques
 t_s_u       = cell(N,1);
for n = 1:N
  t_s_u{n} = cell(Ntr,1);
end
 
 % Spike times repeats
 t_s_r       = cell(N,1);
for n = 1:N
  t_s_r{n} = cell(Ntr,1);
end



%% Calculating processes for repeats and setting random numbers in general
stream_uncommon = RandStream('mrg32k3a','Seed',4);
stream_uncommon_state = stream_uncommon.State;
    
stream_common = RandStream('mt19937ar','Seed',4); 
stream_common_state = stream_common.State;



%% Calculation
tic
parfor n = 1:N
tic_n = tic;
 % Assign Starting values
    ou_signal_u = Bins_Current_signal(randp(P_dist_I_signal,[1,Ntr]));
    ou_signal_r = Bins_Current_signal(randp(P_dist_I_signal,[1,1]));
    ou_noise_u  = Bins_Current_noise (randp(P_dist_I_noise, [1,Ntr]));
    ou_noise_r  = Bins_Current_noise( randp(P_dist_I_noise, [1,Ntr]));  
    
    LI_V_u  = ones(1,Ntr)*LI_V_start;
    LI_V_r  = ones(1,Ntr)*LI_V_start;
 % Assign RandomStreams
    %Uniques
    set(stream_uncommon,'Substream',n);
    %Repeats
    stream_common = RandStream('mt19937ar','Seed',0);
    stream_common.State = stream_common_state;
    t_time = 0;
 for k = 1:length(t)
t_time = t_time+dt;




    % OU-Processes
           
    gauss_signal_u = randn(stream_uncommon, 1,Ntr);
       ou_signal_u = ou_signal_u*ou_decay+ou_mean*(1-ou_decay)+...
                     ou_sigma*sqrt((ou_Tau_cor/2)*(1-ou_decay^2))*gauss_signal_u; 
                 
                 
    gauss_noise_u = randn(stream_uncommon, 1,Ntr);
       ou_noise_u = ou_noise_u*ou_decay+ou_mean*(1-ou_decay)+...
                    ou_sigma*sqrt((ou_Tau_cor/2)*(1-ou_decay^2))*gauss_noise_u;
             
           
    gauss_signal_r  = randn(stream_uncommon,1,1);
       ou_signal_r  = ou_signal_r*ou_decay+ou_mean*(1-ou_decay)+...
                       ou_sigma*sqrt((ou_Tau_cor/2)*(1-ou_decay^2))*gauss_signal_r;


    gauss_noise_r  = randn(stream_uncommon, 1,Ntr);
       ou_noise_r  = ou_noise_r*ou_decay+ou_mean*(1-ou_decay)+...
                     ou_sigma*sqrt((ou_Tau_cor/2)*(1-ou_decay^2))*gauss_noise_r;
     
     % Leaky integrate
        LI_V_u = LI_V_u*LI_decay+(1-LI_decay)*(LI_equi+LI_R*...
                    (sqrt(ou_cor_r)*ou_signal_u+sqrt(1-ou_cor_r)*ou_noise_u));
                
        LI_V_r = LI_V_r*LI_decay+(1-LI_decay)*(LI_equi+LI_R*...
                    (sqrt(ou_cor_r)*repmat(ou_signal_r,1,Ntr) + sqrt(1-ou_cor_r)*ou_noise_r));


   % Spikes Uniques
   spike_yn_u    = (LI_V_u>=ST_threshold);
   index         = find(spike_yn_u);
   % Reset
   LI_V_u(index) = LI_V_reset;
  



   
  for i = index
   t_s_u{n}{i} = [t_s_u{n}{i} t_time]; 
  end
  
     
   % Spikes Repeats
   spike_yn_r = (LI_V_r>=ST_threshold);
   index = find(spike_yn_r);
   % Reset
   LI_V_r(index) = LI_V_reset;
   
  for i = index
   t_s_r{n}{i} = [t_s_r{n}{i} t_time];
  end
  

 end
 
 disp(strcat(['Stimulus ', num2str(n),' of ', num2str(N),' done. Time was ',num2str(toc(tic_n)), ' seconds.']));       
          

 
end
toc

%% Changing order of trials and neurons in the t_s-cells


Spike_count_u = zeros(N,Ntr);
Spike_count_r = zeros(N,Ntr);

for j = 1:N
    for i = 1:Ntr
        Spike_count_u(j,i) = length(t_s_u{j}{i});
        Spike_count_r(j,i) = length(t_s_r{j}{i});
    end
end

Spike_count_0_u = Spike_count_u == 0;
Spike_count_0_r = Spike_count_r == 0;

disp(['UN: There are ', num2str(sum(sum(Spike_count_0_u,2),1)), ' Trials in with 0 Spike count.'])
disp(['UN: That is ', num2str(sum(sum(Spike_count_0_u,2),1)/Ntr/N*100), ' % of all Trials.'])

disp(['RE: There are ', num2str(sum(sum(Spike_count_0_r,2),1)), ' Trials in with 0 Spike count.'])
disp(['RE: That is ', num2str(sum(sum(Spike_count_0_r,2),1)/Ntr/N*100), ' % of all Trials.'])
 % Reordering the cell structure
 t_s_u_new       = cell(N,Ntr);
 t_s_r_new       = cell(N,Ntr);

 % Matrix for first spike only
 t_s_fs_matrix_u = zeros(N,Ntr);
 t_s_fs_matrix_r = zeros(N,Ntr);
 
for j = 1:N
for i = 1:Ntr
  t_s_u_new{j,i} = t_s_u{j}{i};
  t_s_r_new{j,i} = t_s_r{j}{i};
  
  
  if(Spike_count_u(j,i) >= 1)
  t_s_fs_matrix_u(j,i) = t_s_u{j}{i}(1);
  else
  t_s_fs_matrix_u(j,i) = 0;  
  end
  
  if(Spike_count_r(j,i) >= 1)
  t_s_fs_matrix_r(j,i) = t_s_r{j}{i}(1);
  else
  t_s_fs_matrix_r(j,i) = 0;     
  end
  
end
end

  t_s_u = t_s_u_new;
  t_s_r = t_s_r_new;
  
  clear t_s_u_new t_s_r_new


%% Calculating Firing Rates

% FiR for all spikes
FiR_u = Spike_count_u/T_seq_korrigiert*1000;
FiR_r = Spike_count_r/T_seq_korrigiert*1000;

FiR_tr_mean_u = mean(FiR_u,2);
FiR_tr_mean_r = mean(FiR_r,2);

FiR_mean_u = mean(FiR_tr_mean_u);
FiR_mean_r = mean(FiR_tr_mean_r);

% FiR for First-Spike-Matrix
T_fs_tr_mean_u = mean(t_s_fs_matrix_u,2);
T_fs_tr_mean_r = mean(t_s_fs_matrix_r,2);

FiR_fs_tr_mean_u = 1./T_fs_tr_mean_u*1000;
FiR_fs_tr_mean_r = 1./T_fs_tr_mean_r*1000;

T_fs_mean_u = mean(T_fs_tr_mean_u,1);
T_fs_mean_r = mean(T_fs_tr_mean_r,1);

FiR_fs_mean_u = 1./T_fs_mean_u*1000;
FiR_fs_mean_r = 1./T_fs_mean_r*1000;

%% Save


save('params_ISI.mat','stream_uncommon_state', 'stream_common_state',...
     'Ntr', 'N','t','T_*','ou_*', 'LI_*', 'ST*', 'dt');

save('data_ISI.mat','t_s_*', 'FiR_*','-v7.3');


save('first_spike_matrix.mat','t_s_fs_matrix_*', 'FiR*','T_fs*');


end