
dir_return = pwd;



f_name = 'Corr_Readout.mat';



dir_data_Corr = [dir_return,'/Simulation_Data/Corr'];
    
cd(dir_data_Corr);

Dataset_folders = dir('Dataset*');
Dataset_count   = length(Dataset_folders);



ana_plot_yes = true;


if(ana_plot_yes)
%% Analytic

%% Size of Information data

tau_m_start = 1;
tau_m_end   = 100;
tau_m_loop  = 50; %% Resolution of the analytical curve

plot_Tau_ana  = logspace(log10(tau_m_start),log10(tau_m_end),tau_m_loop)+0.001;




plot_Info_ana    = zeros(tau_m_loop,1);
plot_Info_ana_ps = zeros(tau_m_loop,1);
max_inf_freq     = zeros(tau_m_loop,1);
plot_FiR_ana     = zeros(tau_m_loop,1);



%% Constant Parameters


% Input

Tau_cor = 10; %(ms)

R_cor  = 0.6; % Correlation strength between trials
R_auto = 1;   % For Autocorrelation function this value is always 1

ST_threshold = 1; % (mV) 
thresh_ratio = 1;
ou_sigma = ST_threshold./thresh_ratio;




%% Technical Parameters

%Analytics
ana_tau_min =   -200; % (ms)
ana_tau_max =    200; % (ms)
ana_dtau    =    1; % (ms)

trunc_n = 50;




%% Assigning taus

 ana_taus  = ana_tau_min:ana_dtau:ana_tau_max;
 disp(['Number of ana_taus: ',...
        num2str(length(ana_taus))]);
    
%% Assigning frequencies for DFT
ana_delta_f = 1/(ana_tau_max-ana_tau_min)*1000;
ana_freqs   = (0:ana_delta_f:(1/ana_dtau*1000/2))/1000;




%% MAIN - Loops over different paramaters
for k = 1:tau_m_loop

Tau_mem = plot_Tau_ana(k);    

%% Calculating FiR
FiR_ana = 1/(2*pi*sqrt(Tau_cor*Tau_mem))*exp(-((ST_threshold)^2)...
              /(2*(ou_sigma^2*Tau_cor^2/(2*(Tau_cor+Tau_mem)))));




%% Analytic calculation of Autocorrelation function

[autocor_ana, ana_taus_auto]= Cor_analytic(trunc_n,R_auto,ou_sigma,Tau_cor,Tau_mem,...
                               ST_threshold,ST_threshold,ana_taus');


%% Analytic calculation of Crosscorrelation function

crosscor_ana  = Cor_analytic(trunc_n,R_cor,ou_sigma,Tau_cor,Tau_mem,...
                              ST_threshold,ST_threshold,ana_taus');


auto_four_ana  = My_DFT_custom_freqs(ana_taus_auto, autocor_ana', ana_freqs);
auto_four_ana  = real(auto_four_ana);

cross_four_ana = My_DFT_custom_freqs(ana_taus, crosscor_ana', ana_freqs);
cross_four_ana = real(cross_four_ana);

Info_perfreq = -1/2*log2(1-cross_four_ana./(auto_four_ana+1/(ana_tau_max-ana_tau_min)/FiR_ana));

[~, max_inf_freq_index] = max(Info_perfreq(2:end));

max_inf_freq  (k,1) = ana_freqs(max_inf_freq_index+1);
plot_Info_ana (k,1) = sum(Info_perfreq(2:end),2)*ana_delta_f+...
                      Info_perfreq(2)*ana_delta_f; 
plot_Info_ana_ps(k,1) = plot_Info_ana(k,1)/FiR_ana/1000;
plot_FiR_ana    (k,1) = FiR_ana*1000; 


end

end



plot_Tau_Corr     = zeros(Dataset_count,1);
plot_FiR_Corr     = zeros(Dataset_count,1);
plot_Info_Corr    = zeros(Dataset_count,1);
plot_Info_Corr_ps = zeros(Dataset_count,1);


for rr = 1:Dataset_count
    

    
cd(dir_data_Corr);

cd(Dataset_folders(rr,1).name)

load('params.mat','LI_Tau_mem')
plot_Tau_Corr(rr,1) = LI_Tau_mem;
load('data.mat','FiR_u_mean')
plot_FiR_Corr(rr,1)   = FiR_u_mean*1000;
    
  


if(exist(f_name))
loadname = f_name;
load(loadname,'Info_pn_cor2','num_delta_f','FiR_u_mean')

plot_Info_Corr(rr,1) = sum(mean(Info_pn_cor2(:,2:end),1),2)*num_delta_f...
                     + mean(Info_pn_cor2(:,2),1)*num_delta_f;

plot_Info_Corr_ps(rr,1) = plot_Info_Corr(rr,1)/plot_FiR_Corr(rr,1);
else
plot_Info_Corr(rr,1)    = NaN;
plot_Info_Corr_ps(rr,1) = NaN;
end
end



 

f_name = 'Strong_readout.mat';

dir_data_Strong = [dir_return,'/Simulation_Data/Strong'];
    
cd(dir_data_Strong);

Dataset_folders = dir('Dataset*');
Dataset_count   = length(Dataset_folders);




plot_Tau_Strong     = zeros(Dataset_count,1);
plot_FiR_Strong     = zeros(Dataset_count,1);
plot_Info_Strong    = zeros(Dataset_count,1);
plot_Info_Strong_ps = zeros(Dataset_count,1);

for rr = 1:Dataset_count


    
cd(dir_data_Strong);

cd(Dataset_folders(rr,1).name)

loadname = f_name;


load('params.mat','LI_Tau_mem')
plot_Tau_Strong(rr,1) = LI_Tau_mem;
load('data.mat','FiR_u_mean')
plot_FiR_Strong(rr,1) = FiR_u_mean*1000;


load(loadname, 'y_fit')

plot_Info_Strong(rr,1)    = y_fit(1); 
plot_Info_Strong_ps(rr,1) = y_fit(1)/plot_FiR_Strong(rr,1); 


end



fig6a = figure;
set(gca,'FontSize',16)
if(ana_plot_yes)
semilogx(plot_Tau_ana,plot_Info_ana,'r','LineWidth',2)
end
hold on

semilogx(plot_Tau_Corr(:,1),plot_Info_Corr(:,1),'bs','LineWidth',2,'MarkerSize',12)
hold on
% Please note that for numerical reasons the variable plot_Info_Corr can
% have a negligibly small imaginary component, e.g. 0.0000000, because the
% autocorrelation function of the PSTH (c_cross) has a small residual
% asymetry. An autocorrelation function should mathematically be
% symetrical around zero, but whenever both sides -tau and +tau are
% calculated separately, there may be small numerical deviations that lead
% to this residual imaginary component which is multiple orders of
% magnitude smaller than the real value and can safely be neglected.

semilogx(plot_Tau_Strong(:,1),plot_Info_Strong(:,1),'black^','LineWidth',2,'MarkerSize',12)
xlim([3 100]);
title('Information Rate')
xlabel('Membrane Time Constant\tau_{mem} [s]','FontSize',16)
ylabel('Information Rate [bit/(Hz \cdot s)]','FontSize',16)

