
%% Loop parameter
tau_m_start = 1;
tau_m_end   = 100;
tau_m_loop  = 5;

tau_m_vec  = logspace(log10(tau_m_start),log10(tau_m_end),tau_m_loop);
tau_m_vec  = tau_m_vec(2:end);

T_seq_vec =[7500 25000 50000 50000];

Datasetcount = length(tau_m_vec);


tic_total = tic;


% Static parameters
T_SteadyState = 2900; %(ms) 



%% Save directory

dir_name_base = [dir_return,'/Simulation_Data/'];




for xx = 1:Datasetcount
clearvars -except xx tau_m_vec tic_total Datasetcount T_SteadyState T_seq* dir_*
tic_xx = tic;



dir_name = strcat([dir_name_base,'Corr/Dataset', num2str(xx),'/']);



T_sequence    = T_seq_vec(xx)+T_SteadyState; % (ms)


%% Parameter

% Adaptation
ad_Tau   =   5; % (ms)
ad_alpha =  -2; % 
ad_beta  =   4; % 

% Number of stimuli
N = 32;

% Trial count
Ntr = 1000; % 

% OU-Process:
ou_Tau_cor  = 20;%     % (ms) Correlation time of the OU-Process
ou_diffu_const = 1; 
ou_sigma = sqrt(ou_diffu_const); 
ou_mean  = 0;

ou_cor_n =     0; % No noise correlation, noise is always drawn randomly
ou_cor_c =     0; % Independent stimuli for signal entropy trials
ou_cor_r =     0.6; % Cross trial correlation for repeated stimuli

% Leaky Integrator:
LI_Tau_mem = tau_m_vec(xx); % Membrane time constant (ms)
LI_equi    = 0;    % Equilibrium potential (ms)
LI_R       = 1;    % Resistance for Input stimulus 
if(LI_Tau_mem <= ou_Tau_cor)
disp(strcat(['Dataset ', num2str(xx),': Tau_mem is smaller than Tau_ou!!!!!!']))
end
% Spike Generation:
ST_threshold_factor = 0.7;
ST_threshold = LI_equi+ST_threshold_factor*ou_sigma;

LI_V_reset_factor   = 1;
LI_V_reset = LI_equi-LI_V_reset_factor*ou_sigma;

%% Analytic Firing Rate
FiR_analytic = 1/(2*pi*sqrt(ou_Tau_cor*LI_Tau_mem))*exp(-((ST_threshold)^2)...
              /(2*((ou_sigma^2)*ou_Tau_cor^2/(2*(ou_Tau_cor+LI_Tau_mem)))));
          
disp(strcat(['Dataset ', num2str(xx),': Analytically calculated Firing rate is ', num2str(FiR_analytic*1000),'.']));       
          

%% Starting values;
%OU-Process:
ou_start = 0;
% Leaky Integrator:
LI_V_start = LI_equi; % Setting equilibrium as start




%% Assigning timestep size, stimestep count and time vector
   dt = 0.05; 
steps = ceil(T_sequence/dt);

disp(strcat(['Dataset ', num2str(xx),': Number of timesteps is ', num2str(steps),'.']));       


T_seq_korrigiert = steps*dt;




%% Assign Help functions

%Decays for OU and LI process
ou_decay = exp(-dt/ou_Tau_cor);
LI_decay = exp(-dt/LI_Tau_mem);

ad_decay = exp(-dt/ad_Tau);

% Coefficients uniques
Au_tn = sqrt((1-ou_cor_n)*(1-ou_cor_c)+ou_cor_n);
Au_t  = sqrt((1-ou_cor_n)*ou_cor_c);

% Coefficients repeats
Ar_tn = sqrt((1-ou_cor_n)*(1-ou_cor_c)*(1-ou_cor_r)+ou_cor_n*(1-ou_cor_r));
Ar_t  = sqrt((1-ou_cor_n)*ou_cor_c*(1-ou_cor_r));
Ar_n  = sqrt((1-ou_cor_n)*(1-ou_cor_c)*ou_cor_r+ou_cor_n*ou_cor_r);
Ar_0  = sqrt((1-ou_cor_n)*ou_cor_c*ou_cor_r);




%% Assign Vectors
% Spike times uniques
 t_s_u       = cell(N,1);
for n = 1:N
  t_s_u{n} = cell(Ntr,1);
end
 
 % Spike times repeats
 t_s_r       = cell(N,1);
for n = 1:N
  t_s_r{n} = cell(Ntr,1);
end



%% Calculating processes, setting the seeds for random numbers
stream_uncommon_seed = 4;
stream_uncommon = RandStream('mrg32k3a','Seed',stream_uncommon_seed);
stream_uncommon_state = stream_uncommon.State;

stream_common_seed = 4;
stream_common = RandStream('mt19937ar','Seed',stream_common_seed); 
stream_common_state = stream_common.State;


%% Calculation
tic_calculation = tic;
parfor n = 1:N
tic_n = tic;
 % Assign Starting values
    ou_tn = ones(1,Ntr)*ou_start;
    ou_t  = ones(1,Ntr)*ou_start;  
    ou_n  =             ou_start;
    ou_0  =             ou_start;  
    
    LI_V_u  = ones(1,Ntr)*LI_V_start;
    LI_V_r  = ones(1,Ntr)*LI_V_start;
    
    ad_u = zeros(1,Ntr);
    ad_r = zeros(1,Ntr);
    
 % Assign RandomStreams
    %Uniques
    set(stream_uncommon,'Substream',n);
    %Repeats
    stream_common = RandStream('mt19937ar','Seed',0);
    stream_common.State = stream_common_state;
 
 t_time = 0;
 for k = 1:steps

     
     t_time = t_time+dt;
     

    
    
    % OU-Processes
    gauss_tn = randn(stream_uncommon, 1,Ntr);
       ou_tn = ou_tn*ou_decay+ou_mean*(1-ou_decay)+...
               ou_sigma*sqrt((ou_Tau_cor/2)*(1-ou_decay^2))*gauss_tn;
           
           
    gauss_n  = randn(stream_uncommon, 1,1);
       ou_n  = ou_n*ou_decay+ou_mean*(1-ou_decay)+...
               ou_sigma*sqrt((ou_Tau_cor/2)*(1-ou_decay^2))*gauss_n;
             
           
    gauss_t  = randn(stream_common,1,Ntr);
       ou_t  = ou_t*ou_decay+ou_mean*(1-ou_decay)+...
               ou_sigma*sqrt((ou_Tau_cor/2)*(1-ou_decay^2))*gauss_t;


    gauss_0  = randn(stream_common,1,1);
       ou_0  = ou_0*ou_decay+ou_mean*(1-ou_decay)+...
               ou_sigma*sqrt((ou_Tau_cor/2)*(1-ou_decay^2))*gauss_0;
     
    % Leaky integrate
                
        LI_V_u = LI_V_u*LI_decay+(1-LI_decay)*(LI_equi+LI_R*...
                    (ad_alpha*ad_u + Au_tn*ou_tn+Au_t*ou_t));
                
        LI_V_r = LI_V_r*LI_decay+(1-LI_decay)*(LI_equi+LI_R*...
                    (ad_alpha*ad_r + Ar_tn*ou_tn + Ar_t*ou_t+...
                     Ar_n *repmat(ou_n,1,Ntr)  + Ar_0*repmat(ou_0,1,Ntr)));
                 
                 
     % Adaptation
        ad_u = ad_u*ad_decay  +  (1-ad_decay)*ad_beta*LI_V_u;  
        ad_r = ad_r*ad_decay  +  (1-ad_decay)*ad_beta*LI_V_r;          
           
   % Spikes Uniques
   spike_yn_u    = (LI_V_u>=ST_threshold);
   index         = find(spike_yn_u);
   % Reset
   LI_V_u(index) = LI_V_reset;
  
  if(t_time >= T_SteadyState)
   for i = index
    t_s_u{n}{i} = [t_s_u{n}{i} (t_time-T_SteadyState)];
   end
  end
  
   % Spikes Repeats
   spike_yn_r = (LI_V_r>=ST_threshold);
   index = find(spike_yn_r);
   % Reset
   LI_V_r(index) = LI_V_reset;
   
   
  if(t_time >= T_SteadyState)
   for i = index
    t_s_r{n}{i} = [t_s_r{n}{i} (t_time-T_SteadyState)];
   end
  end

 
 end


 
 
 disp(strcat(['Dataset ', num2str(xx),': Stimulus ', num2str(n),' of ', num2str(N),' done. Time was ',num2str(toc(tic_n)), ' seconds.']));       
          
end


disp(strcat(['Dataset ', num2str(xx),': Calculated. Time was ',num2str(toc(tic_calculation)), ' seconds.'])); 

%% Accounting for steady state

T_seq_korrigiert = T_seq_korrigiert - T_SteadyState;

%% Calculating Firing Rates
tic_FiR = tic;

FiR_u = zeros(N,Ntr);
FiR_r = zeros(N,Ntr);
parfor i = 1:N
for j = 1:Ntr
   FiR_u(i,j) = length(t_s_u{i}{j})/T_seq_korrigiert;
   FiR_r(i,j) = length(t_s_r{i}{j})/T_seq_korrigiert;
end
end
FiR_u_mean = mean(mean(FiR_u,2),1);
FiR_r_mean = mean(mean(FiR_r,2),1);

disp(strcat(['Dataset ', num2str(xx),': FiR calculated. Time was ',num2str(toc(tic_FiR)), ' seconds.']));


tic_saving = tic;


%% Save
R = dir(dir_name);
if (isempty(R))
    mkdir(dir_name);
end

cd(dir_name)

saving_time = toc(tic_calculation);

clear R dir_name

save('params.mat','stream_uncommon_state*', 'stream_common_state*',...
     'Ntr', 'N','T_*', 'ou_*', 'LI_*', 'ST*', 'dt');

save('data.mat','t_s_*', 'FiR*','saving_time','-v7.3');


disp(strcat(['Dataset ', num2str(xx),': Saved. Time was ',num2str(toc(tic_saving)), ' seconds.']));  
disp(strcat(['Dataset ', num2str(xx),' of ', num2str(Datasetcount),' done!.' ]));

cd ../..

end


disp(strcat(['COMPLETE SIMULATION FINISHED. Time was ',num2str(toc(tic_total)), ' seconds.']));



